from keras.callbacks import Callback
from _thread import start_new_thread

from notify_run import Notify


class NotifyCallback(Callback):
    def __init__(self, notify=None):
        if notify is None:
            notify = Notify()
        self.notify = notify
        self._format_pair = '{}: {:.2f}'
    
    def _format_stats(self, logs):
        return ''.join(self._format_pair.format(k, v) for k, v in logs.items())

    def send_message(self, message):
        start_new_thread(self.notify.send, (message,))
    
    def on_train_begin(self, logs=None):
        self.epochs = self.params['epochs']

    def on_epoch_end(self, epoch, logs=None):
        epoch += 1
        if epoch == self.epochs:
            message = 'Done training {}'.format(self.epochs)
        else:
            message = 'Done epoch {} of {}.'.format(epoch, self.epochs)
        message += ' ' + self._format_stats(logs) if logs else ''
        self.send_message(message)




import argparse

import requests

from notify_run import Notify, NotConfigured


def check_existing(notify, force):
    if notify.config_file_exists and not force:
        print(
            'Overwrite existing configuration ({})? [y/N]'.format(notify.endpoint))

        try:
            # Python 2.x
            input = raw_input
        except:
            raise

        return input().lower() == 'y'
    return True


def configure(notify, force):
    if check_existing(notify, force):
        notify.write_config()


def info(notify):
    print(notify.info())


def register(notify, force):
    if check_existing(notify, force):
        print(notify.register())


def send(notify, message):
    notify.send(message)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--api-server', '-a')

    subparsers = parser.add_subparsers()

    # "send" command: send a message
    notify_parser = subparsers.add_parser('send')
    notify_parser.add_argument('message')
    notify_parser.add_argument('--endpoint', '-e')
    notify_parser.set_defaults(func=send)

    # "configure" command: configure to an existing endpoint
    conf_parser = subparsers.add_parser('configure')
    conf_parser.add_argument('endpoint')
    conf_parser.add_argument('--force', '-f', action='store_true')
    conf_parser.set_defaults(func=configure)

    # "info" command: get information for subscribing to an endpoint
    info_parser = subparsers.add_parser('info')
    info_parser.add_argument('--endpoint', '-e')
    info_parser.set_defaults(func=info)

    # "register" command: register a new endpoint
    register_parser = subparsers.add_parser('register')
    register_parser.add_argument('--force', '-f', action='store_true')
    register_parser.set_defaults(func=register)

    args = vars(parser.parse_args())
    endpoint = None
    if 'endpoint' in args:
        endpoint = args.pop('endpoint')
    if 'func' in args:
        func = args.pop('func')
    else:
        parser.print_usage()
        return
    api_server = args.pop('api_server')

    try:
        notify = Notify(api_server, endpoint)
        func(notify, **args)
    except NotConfigured:
        print('Run "notify-run register" or "notify-run configure <endpoint>" first.')
    except ValueError as ve:
        print(ve)
    except requests.exceptions.ConnectionError as ce:
        print('Error connecting. Check configuration and internet connection.')
        print(ce)



from os import environ, makedirs
from os.path import expanduser, dirname
import json
import requests
from pyqrcode import QRCode
from io import BytesIO

try:
    # Python 3.x
    from urllib.parse import urlparse
except ImportError:
    # Python 2.x
    from urlparse import urlparse

DEFAULT_API_SERVER = environ.get(
    'NOTIFY_API_SERVER', 'https://notify.run/api/')
CONFIG_FILENAME = '~/.config/notify-run'


class NotConfigured(Exception):
    pass


class EndpointInfo:
    def __init__(self, info_dict):
        self.endpoint = info_dict['endpoint']
        self.channel_page = info_dict['channel_page']

    def _qr(self):
        return QRCode(self.channel_page, error='L')

    def _qr_svg(self):
        buffer = BytesIO()
        self._qr().svg(buffer, 6)
        return buffer.getvalue().decode('utf-8')

    def __repr__(self):
        return '''Endpoint: {}
To subscribe, open: {}
Or scan this QR code:
{}
        '''.format(
            self.endpoint,
            self.channel_page,
            self._qr().terminal(quiet_zone=1)
        )

    def _repr_html_(self):
        return '''<p>Endpoint: <samp>{0}</samp></p>
<p>To subscribe, open: <a href="{1}">{1}</a></p>
<p>Or scan this QR code:</p>
{2}
        '''.format(
            self.endpoint,
            self.channel_page,
            self._qr_svg()
        )


class Notify:
    def __init__(self, api_server=None, endpoint=None):
        self.api_server = api_server or DEFAULT_API_SERVER
        self.endpoint = None
        self._config_file = expanduser(CONFIG_FILENAME)

        self.read_config()

        if endpoint is not None:
            parse_result = urlparse(endpoint)
            if parse_result.scheme not in ['http', 'https']:
                raise ValueError('Endpoint should be a URL with an HTTP or HTTPS scheme (scheme was {}.)'.format(
                    parse_result.scheme))
            self.endpoint = endpoint

    # Config

    def read_config(self):
        self.config_file_exists = False
        try:
            with open(self._config_file, 'r') as conffile:
                self.config_file_exists = True
                config = json.load(conffile)
                self.endpoint = config['endpoint']
        except (IOError, OSError):
            return
        except ValueError:
            print('Invalid JSON in {}'.format(self._config_file))
            return

    def write_config(self):
        try:
            makedirs(dirname(self._config_file))
        except OSError:
            # file exists
            pass

        config = {
            'endpoint': self.endpoint,
        }
        try:
            with open(self._config_file, 'w') as conffile:
                json.dump(config, conffile)
        except Exception:
            raise

    # Commands

    def send(self, message):
        if self.endpoint is None:
            raise NotConfigured()
        requests.post(self.endpoint, message)

    def info(self):
        if self.endpoint is None:
            raise NotConfigured
        r = requests.get(self.endpoint + '/info').json()
        return EndpointInfo(r)

    def register(self):
        r = requests.post(self.api_server + 'register_channel').json()
        self.endpoint = r['endpoint']
        self.write_config()
        return EndpointInfo(r)



from setuptools import setup


def readme():
    with open('README.rst') as f:
        return f.read()


setup(name='notify-run',
      version='0.0.10',
      python_requires='>=2.7',
      description='Client for notify.run notifications.',
      long_description=readme(),
      author='Paul Butler',
      author_email='notify@paulbutler.org',
      url='https://notify.run/',
      packages=['notify_run'],
      entry_points={
          'console_scripts': [
              'notify-run = notify_run.cli:main'
          ]
      },
      install_requires=[
          'PyQRCode==1.2.1',
          'requests==2.18.4',
      ],
      )
